@@ -30,9 +30,6 @@ from firewall.core import ipXtables # some common stuff lives there
 from firewall.errors import FirewallError, INVALID_IPV
 import string
 
-PROC_IPxTABLE_NAMES = {
-}
-
 BUILT_IN_CHAINS = {
     "broute": [ "BROUTING" ],
     "nat": [ "PREROUTING", "POSTROUTING", "OUTPUT" ],
@@ -83,7 +80,7 @@ class ebtables(object):
         # with --noflush
         rules = [ ]
         try:
-            self.set_rules(rules, flush=False)
+            self.set_rules(rules, "off")
         except ValueError:
             return False
         return True
@@ -140,7 +137,7 @@ class ebtables(object):
     def reverse_passthrough(self, args):
         return ipXtables.common_reverse_passthrough(args)
 
-    def set_rules(self, rules, flush=False, log_denied="off"):
+    def set_rules(self, rules, log_denied):
         temp_file = tempFile()
 
         table = "filter"
@@ -183,8 +180,7 @@ class ebtables(object):
         log.debug2("%s: %s %s", self.__class__, self._restore_command,
                    "%s: %d" % (temp_file.name, stat.st_size))
         args = [ ]
-        if not flush:
-            args.append("--noflush")
+        args.append("--noflush")
 
         (status, ret) = runProg(self._restore_command, args,
                                 stdin=temp_file.name)
@@ -206,7 +202,7 @@ class ebtables(object):
                                                      " ".join(args), ret))
         return ret
 
-    def set_rule(self, rule, log_denied="off"):
+    def set_rule(self, rule, log_denied):
         self._rule_validate(rule)
         return self.__run(rule)
 
@@ -229,12 +225,9 @@ class ebtables(object):
     def get_zone_table_chains(self, table):
         return {}
 
-    def used_tables(self):
-        return list(BUILT_IN_CHAINS.keys())
-
     def build_flush_rules(self):
         rules = []
-        for table in self.used_tables():
+        for table in BUILT_IN_CHAINS.keys():
             # Flush firewall rules: -F
             # Delete firewall chains: -X
             # Set counter to zero: -Z
@@ -242,14 +235,9 @@ class ebtables(object):
                 rules.append(["-t", table, flag])
         return rules
 
-    def build_set_policy_rules(self, policy, which="used"):
-        if which == "used":
-            tables = self.used_tables()
-        else:
-            tables = list(BUILT_IN_CHAINS.keys())
-
+    def build_set_policy_rules(self, policy):
         rules = []
-        for table in tables:
+        for table in BUILT_IN_CHAINS.keys():
             for chain in BUILT_IN_CHAINS[table]:
                 rules.append(["-t", table, "-P", chain, policy])
         return rules
