@@ -829,7 +829,7 @@ class Firewall(object):
         if use_transaction is None:
             transaction.execute(True)
 
-    def set_policy(self, policy, which="used", use_transaction=None):
+    def set_policy(self, policy, use_transaction=None):
         if use_transaction is None:
             transaction = FirewallTransaction(self)
         else:
@@ -838,7 +838,7 @@ class Firewall(object):
         log.debug1("Setting policy to '%s'", policy)
 
         for backend in self.enabled_backends():
-            rules = backend.build_set_policy_rules(policy, which)
+            rules = backend.build_set_policy_rules(policy)
             transaction.add_rules(backend, rules)
 
         if use_transaction is None:
@@ -847,6 +847,9 @@ class Firewall(object):
     # rule function used in handle_ functions
 
     def rule(self, backend_name, rule):
+        if not rule:
+            return ""
+
         backend = self.get_backend_by_name(backend_name)
         if not backend:
             raise FirewallError(errors.INVALID_IPV,
@@ -858,7 +861,7 @@ class Firewall(object):
         return backend.set_rule(rule, self._log_denied)
 
     def rules(self, backend_name, rules):
-        _rules = rules[:]
+        _rules = list(filter(None, rules))
 
         backend = self.get_backend_by_name(backend_name)
         if not backend:
@@ -873,21 +876,21 @@ class Firewall(object):
            (backend_name == "ebtables" and not self.ebtables_backend.restore_noflush_option):
             for i,rule in enumerate(_rules):
                 try:
-                    backend.set_rule(rule)
+                    backend.set_rule(rule, self._log_denied)
                 except Exception as msg:
                     log.debug1(traceback.format_exc())
                     log.error("Failed to apply rules. A firewall reload might solve the issue if the firewall has been modified using ip*tables or ebtables.")
                     log.error(msg)
                     for rule in reversed(_rules[:i]):
                         try:
-                            backend.set_rule(backend.reverse_rule(rule))
+                            backend.set_rule(backend.reverse_rule(rule), self._log_denied)
                         except Exception:
                             # ignore errors here
                             pass
                     return False
             return True
         else:
-            return backend.set_rules(_rules, log_denied=self._log_denied)
+            return backend.set_rules(_rules, self._log_denied)
 
     # check functions
 
@@ -1023,7 +1026,7 @@ class Firewall(object):
 
         # TODO: use rule in raw table not default chain policy
         try:
-            self.set_policy("DROP", "all")
+            self.set_policy("DROP")
         except Exception as msg:
             raise FirewallError(errors.COMMAND_FAILED, msg)
         self._panic = True
@@ -1035,7 +1038,7 @@ class Firewall(object):
 
         # TODO: use rule in raw table not default chain policy
         try:
-            self.set_policy("ACCEPT", "all")
+            self.set_policy("ACCEPT")
         except Exception as msg:
             raise FirewallError(errors.COMMAND_FAILED, msg)
         self._panic = False
