@@ -31,11 +31,6 @@ from firewall.errors import FirewallError, INVALID_PASSTHROUGH, INVALID_RULE
 from firewall.core.rich import Rich_Accept, Rich_Reject, Rich_Drop, Rich_Mark
 import string
 
-PROC_IPxTABLE_NAMES = {
-    "ipv4": "/proc/net/ip_tables_names",
-    "ipv6": "/proc/net/ip6_tables_names",
-}
-
 BUILT_IN_CHAINS = {
     "security": [ "INPUT", "OUTPUT", "FORWARD" ],
     "raw": [ "PREROUTING", "OUTPUT" ],
@@ -156,7 +151,7 @@ OUR_CHAINS["filter"] = set(["INPUT_direct", "INPUT_ZONES_SOURCE", "INPUT_ZONES",
 
 # ipv ebtables also uses this
 #
-def common_reverse_rule(self, args):
+def common_reverse_rule(args):
     """ Inverse valid rule """
 
     replace_args = {
@@ -390,7 +385,7 @@ class ip4tables(object):
                     chain = args[i+1]
         return (table, chain)
 
-    def set_rules(self, rules, flush=False, log_denied="off"):
+    def set_rules(self, rules, log_denied):
         temp_file = tempFile()
 
         table_rules = { }
@@ -458,8 +453,7 @@ class ip4tables(object):
         args = [ ]
         if self.restore_wait_option:
             args.append(self.restore_wait_option)
-        if not flush:
-            args.append("-n")
+        args.append("-n")
 
         (status, ret) = runProg(self._restore_command, args,
                                 stdin=temp_file.name)
@@ -481,7 +475,7 @@ class ip4tables(object):
                                                      " ".join(args), ret))
         return ret
 
-    def set_rule(self, rule, log_denied="off"):
+    def set_rule(self, rule, log_denied):
         # replace %%REJECT%%
         self._rule_replace(rule, "%%REJECT%%", \
                 ["REJECT", "--reject-with", DEFAULT_REJECT_TYPE[self.ipv]])
@@ -521,19 +515,6 @@ class ip4tables(object):
 
         return ret
 
-    def used_tables(self):
-        tables = [ ]
-        filename = PROC_IPxTABLE_NAMES[self.ipv]
-
-        if os.path.exists(filename):
-            with open(filename, "r") as f:
-                for line in f.readlines():
-                    if not line:
-                        break
-                    tables.append(line.strip())
-
-        return tables
-
     def _detect_wait_option(self):
         wait_option = ""
         ret = runProg(self._command, ["-w", "-L", "-n"])  # since iptables-1.4.20
@@ -567,7 +548,7 @@ class ip4tables(object):
 
     def build_flush_rules(self):
         rules = []
-        for table in self.used_tables():
+        for table in BUILT_IN_CHAINS.keys():
             # Flush firewall rules: -F
             # Delete firewall chains: -X
             # Set counter to zero: -Z
@@ -575,14 +556,9 @@ class ip4tables(object):
                 rules.append(["-t", table, flag])
         return rules
 
-    def build_set_policy_rules(self, policy, which="used"):
-        if which == "used":
-            tables = self.used_tables()
-        else:
-            tables = list(BUILT_IN_CHAINS.keys())
-
+    def build_set_policy_rules(self, policy):
         rules = []
-        for table in tables:
+        for table in BUILT_IN_CHAINS.keys():
             if table == "nat":
                 continue
             for chain in BUILT_IN_CHAINS[table]:
@@ -787,9 +763,9 @@ class ip4tables(object):
         rule = [ add_del, "%s_log" % (target), "-t", table]
         rule += rule_fragment + [ "-j", "LOG" ]
         if rich_rule.log.prefix:
-            rule += [ "--log-prefix", '"%s"' % rich_rule.log.prefix ]
+            rule += [ "--log-prefix", "'%s'" % rich_rule.log.prefix ]
         if rich_rule.log.level:
-            rule += [ "--log-level", '"%s"' % rich_rule.log.level ]
+            rule += [ "--log-level", "%s" % rich_rule.log.level ]
         rule += self._rule_limit(rich_rule.log.limit)
 
         return rule
@@ -992,9 +968,8 @@ class ip4tables(object):
                                             zone=zone)
         rule_fragment = []
         if rich_rule:
-            # note the reversed source/destination
-            rule_fragment += self._rich_rule_destination_fragment(rich_rule.source)
-            rule_fragment += self._rich_rule_source_fragment(rich_rule.destination)
+            rule_fragment += self._rich_rule_destination_fragment(rich_rule.destination)
+            rule_fragment += self._rich_rule_source_fragment(rich_rule.source)
 
         rules.append([ add_del, "%s_allow" % (target), "-t", "filter"]
                      + rule_fragment +
@@ -1097,10 +1072,10 @@ class ip4tables(object):
         return rules
 
     def build_zone_icmp_block_inversion_rules(self, enable, zone):
-        rule_idx = 4
         table = "filter"
         rules = []
         for chain in [ "INPUT", "FORWARD_IN" ]:
+            rule_idx = 4
             _zone = DEFAULT_ZONE_TARGET.format(chain=SHORTCUTS[chain],
                                                zone=zone)
 
