@@ -166,9 +166,6 @@ class nftables(object):
         self.restore_command_exists = False
 
     def __run(self, args):
-        if not args:
-            return ""
-
         nft_opts = ["--echo", "--handle"]
         _args = args[:]
 
@@ -259,7 +256,7 @@ class nftables(object):
         ret_args[0] = "delete"
         return ret_args
 
-    def set_rules(self, rules, flush=False, log_denied="off"):
+    def set_rules(self, rules, log_denied):
         # We can't support using "nft -f" because we need to retrieve the
         # handles for each rules so we can delete them later on.
         # See also: self.restore_command_exists
@@ -268,7 +265,7 @@ class nftables(object):
         #
         raise FirewallError(UNKNOWN_ERROR, "not implemented")
 
-    def set_rule(self, rule, log_denied="off"):
+    def set_rule(self, rule, log_denied):
         # replace %%REJECT%%
         #
         # HACK: work around nft bug in which icmpx does not work if the rule
@@ -312,7 +309,7 @@ class nftables(object):
             rules.append(["delete", "table", family, "%s" % TABLE_NAME])
         return rules
 
-    def build_set_policy_rules(self, policy, which="used"):
+    def build_set_policy_rules(self, policy):
         # Policy is not exposed to the user. It's only to make sure we DROP
         # packets while initially starting and for panic mode. As such, using
         # hooks with a higher priority than our base chains is sufficient.
@@ -603,13 +600,14 @@ class nftables(object):
         if self._fw.get_log_denied() != "off":
             if table == "filter" and \
                chain in ["INPUT", "FORWARD_IN", "FORWARD_OUT", "OUTPUT"]:
-                log_suffix = "DROP"
-                if target in ["REJECT", "%%REJECT%%"]:
-                    log_suffix = "REJECT"
-                rules.append(["add", "rule", family, "%s" % TABLE_NAME,
-                              "%s_%s" % (table, _zone), "type", "%%LOGTYPE%%",
-                              "log", "prefix",
-                              "\"filter_%s_%s: \"" % (_zone, log_suffix)])
+                if target in ["REJECT", "%%REJECT%%", "DROP"]:
+                    log_suffix = target
+                    if target == "%%REJECT%%":
+                        log_suffix = "REJECT"
+                    rules.append(["add", "rule", family, "%s" % TABLE_NAME,
+                                  "%s_%s" % (table, _zone), "%%LOGTYPE%%",
+                                  "log", "prefix",
+                                  "\"filter_%s_%s: \"" % (_zone, log_suffix)])
 
         # Handle trust, block and drop zones:
         # Add an additional rule with the zone target (accept, reject
