@@ -25,15 +25,17 @@ __all__ = [ "PY2", "getPortID", "getPortRange", "portStr", "getServiceName",
             "firewalld_is_active", "tempFile", "readfile", "writefile",
             "enable_ip_forwarding", "get_nf_conntrack_helper_setting",
             "set_nf_conntrack_helper_setting", "get_nf_conntrack_helpers",
-            "check_port", "check_address",
+            "get_nf_nat_helpers", "check_port", "check_address",
             "check_single_address", "check_mac", "uniqify", "ppid_of_pid",
             "max_zone_name_len", "checkUser", "checkUid", "checkCommand",
             "checkContext", "joinArgs", "splitArgs",
             "b2u", "u2b", "u2b_if_py2" ]
 
 import socket
-import os, os.path
-import shlex, pipes
+import os
+import os.path
+import shlex
+import pipes
 import string
 import sys
 import tempfile
@@ -182,14 +184,16 @@ def checkIPnMask(ip):
     if "/" in ip:
         addr = ip[:ip.index("/")]
         mask = ip[ip.index("/")+1:]
+        if len(addr) < 1 or len(mask) < 1:
+            return False
     else:
         addr = ip
         mask = None
     if not checkIP(addr):
         return False
     if mask:
-        if "." in mask and checkIP(addr):
-            return False
+        if "." in mask:
+            return checkIP(mask)
         else:
             try:
                 i = int(mask)
@@ -203,6 +207,8 @@ def checkIP6nMask(ip):
     if "/" in ip:
         addr = ip[:ip.index("/")]
         mask = ip[ip.index("/")+1:]
+        if len(addr) < 1 or len(mask) < 1:
+            return False
     else:
         addr = ip
         mask = None
@@ -339,6 +345,16 @@ def get_nf_conntrack_helpers():
             (status, ret) = runProg(COMMANDS["modinfo"], [ module, ])
             if status != 0:
                 continue
+            # If module name matches "nf_conntrack_proto_*"
+            # the we add it to helpers list and goto next module
+            if filename.startswith("nf_conntrack_proto_"):
+                helper = filename.split(".")[0].strip()
+                helper = helper.replace("_", "-")
+                helper = helper.replace("nf-conntrack-", "")
+                helpers.setdefault(module, [ ]).append(helper)
+                continue
+            # Else we get module alias and if "-helper" in the "alias:" line of modinfo
+            # then we add it to helpers list and goto next module
             for line in ret.split("\n"):
                 if line.startswith("alias:") and "-helper-" in line:
                     helper = line.split(":")[1].strip()
@@ -347,6 +363,37 @@ def get_nf_conntrack_helpers():
                     helpers.setdefault(module, [ ]).append(helper)
     return helpers
 
+def get_nf_nat_helpers():
+    kver = os.uname()[2]
+    helpers = { }
+    for path in ["/lib/modules/%s/kernel/net/netfilter/" % kver,
+                 "/lib/modules/%s/kernel/net/ipv4/netfilter/" % kver,
+                 "/lib/modules/%s/kernel/net/ipv6/netfilter/" % kver]:
+        if os.path.isdir(path):
+            for filename in sorted(os.listdir(path)):
+                if not filename.startswith("nf_nat_"):
+                    continue
+                module = filename.split(".")[0]
+                (status, ret) = runProg(COMMANDS["modinfo"], [ module, ])
+                if status != 0:
+                    continue
+                # If module name matches "nf_nat_proto_*"
+                # the we add it to helpers list and goto next module
+                if filename.startswith("nf_nat_proto_"):
+                    helper = filename.split(".")[0].strip()
+                    helper = helper.replace("_", "-")
+                    helper = helper.replace("nf-nat-", "")
+                    helpers.setdefault(module, [ ]).append(helper)
+                    continue
+                # Else we get module alias and if "NAT helper" in "description:" line of modinfo
+                # then we add it to helpers list and goto next module
+                for line in ret.split("\n"):
+                    if line.startswith("description:") and "NAT helper" in line:
+                        helper = module.replace("nf_nat_", "")
+                        helper = helper.replace("_", "-")
+                        helpers.setdefault(module, [ ]).append(helper)
+    return helpers
+
 def get_nf_conntrack_helper_setting():
     try:
         return int(readfile("/proc/sys/net/netfilter/nf_conntrack_helper")[0])
@@ -484,7 +531,7 @@ def joinArgs(args):
         return " ".join(pipes.quote(a) for a in args)
 
 def splitArgs(_string):
-    if PY2 and isinstance(_string, unicode):
+    if PY2 and isinstance(_string, unicode): # noqa: F821
         # Python2's shlex doesn't like unicode
         _string = u2b(_string)
         splits = shlex.split(_string)
@@ -506,6 +553,6 @@ def u2b(_string):
 
 def u2b_if_py2(_string):
     """ unicode to bytes only if Python 2"""
-    if PY2 and isinstance(_string, unicode):
+    if PY2 and isinstance(_string, unicode): # noqa: F821
         return _string.encode('UTF-8', 'replace')
     return _string
